## This file is part of mlpy.
## IUWT
   
## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2010 Fondazione Bruno Kessler - Via Santa Croce 77, 38100 Trento, ITALY.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['iuwt_haar']

import numpy as np
import irregularuwt
from  _extend import extend


def iuwt_haar(x, y, levels=0):
    """    
    Irregular Undecimated Wavelet Tranform
    
    :Parameters:
      x : 1d numpy array
        sorted locations
      y : 1d numpy array
        observations
      levels : integer
            maximum level of the decomposition (J).
            If levels = 0 computes all levels.

    :Returns:
      s, d : 2d numpy array (J x len(x)),
             2d numpy array (J x len(x))
        scaling coefficients (smooth signal),
        wavelet coefficients (detail signal)
    """


    ye = extend(y, method='reflection', length='powerof2')
    xe = np.append(x, np.arange(x[-1]+1,
        x[-1]+ye.shape[0]-x.shape[0]+1))
    
    s, d, support = irregularuwt.haar(xe, ye)
    
    for i in np.arange(s.shape[0]):
        s[i] = np.roll(s[i], 2**i)
        d[i] = np.roll(d[i], 2**i)
        
    s = s[:, 0:x.shape[0]]
    d = d[:, 0:x.shape[0]]

    return s, d

